/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.brooklyn.core.mgmt.rebind;

import java.util.Collection;
import java.util.Map;

import org.apache.brooklyn.api.mgmt.rebind.RebindContext;
import org.apache.brooklyn.api.mgmt.rebind.mementos.PolicyMemento;
import org.apache.brooklyn.config.ConfigKey;
import org.apache.brooklyn.core.entity.internal.ConfigUtilsInternal;
import org.apache.brooklyn.core.policy.AbstractPolicy;
import org.apache.brooklyn.util.core.config.ConfigBag;
import org.apache.brooklyn.util.core.flags.FlagUtils;

public class BasicPolicyRebindSupport extends AbstractBrooklynObjectRebindSupport<PolicyMemento> {

    private final AbstractPolicy policy;
    
    public BasicPolicyRebindSupport(AbstractPolicy policy) {
        super(policy);
        this.policy = policy;
    }
    
    @Override
    protected void addConfig(RebindContext rebindContext, PolicyMemento memento) {
        // TODO entity does config-lookup differently; the memento contains the config keys.
        // BasicEntityMemento.postDeserialize uses the injectTypeClass to call EntityTypes.getDefinedConfigKeys(clazz)
        
        Collection<ConfigKey<?>> configKeys = policy.getAdjunctType().getConfigKeys();
        
        Map<?, ?> flags = memento.getConfig();
        
        // First set the config keys that are known explicitly (including with deprecated names).
        flags = ConfigUtilsInternal.setAllConfigKeys(flags, configKeys, policy);

        // Note that the flags may have been set in the constructor; but non-legacy policies should have no-arg constructors
        if (!flags.isEmpty()) {
            ConfigBag configBag = ConfigBag.newInstance(flags);
            FlagUtils.setFieldsFromFlags(policy, configBag);
            FlagUtils.setAllConfigKeys(policy, configBag, false);
        }
    }

    @Override
    protected void addCustoms(RebindContext rebindContext, PolicyMemento memento) {
        // no-op
    }
}
