/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.brooklyn.test.http;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.brooklyn.util.exceptions.Exceptions;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpException;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicHeader;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpRequestHandler;

public class TestHttpRequestHandler implements HttpRequestHandler {
    private HttpEntity entity;
    private int responseCode = HttpStatus.SC_OK;
    private Collection<Header> headers = new ArrayList<Header>();

    public TestHttpRequestHandler response(String response) {
        try {
            this.entity = new StringEntity(response);
        } catch (UnsupportedEncodingException e) {
            throw Exceptions.propagate(e);
        }
        return this;
    }

    public TestHttpRequestHandler response(byte[] response) {
        this.entity = new ByteArrayEntity(response);
        return this;
    }

    public TestHttpRequestHandler code(int responseCode) {
        this.responseCode = responseCode;
        return this;
    }

    public TestHttpRequestHandler header(String name, String value) {
        headers.add(new BasicHeader(name, value));
        return this;
    }

    @Override
    public void handle(HttpRequest request, HttpResponse response, HttpContext context) throws HttpException, IOException {
        for (Header h : headers) {
            response.setHeader(h);
        }

        response.setStatusCode(responseCode);
        response.setEntity(entity);
    }

}
